@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-RCSID="$Id: hostbup.py,v 1.46 2007/04/26 15:38:00 andreas Exp $"
+RCSID="$Id: hostbup.py,v 1.52 2008/06/25 13:51:37 andreas Exp $"
 
 import shlex, sys, string, os, pwd, getopt, time, types, time, smtplib
 import thread, Queue, signal, select, statvfs, resource
@@ -169,21 +169,22 @@ class Uulock:
 			self.LF=""
 	
 	def __init__(self, pid, resource, lockfile="/var/spool/lock/LCK"):
-		LFtmp=lockfile+'..%d' % pid
-		self.LF=lockfile+'..%s' % resource
+		self.LF=lockfile+'.%s' % resource
+		LFtmp=self.LF+'.%d' % pid
 		try:
 			lf=open(LFtmp,'w')
 		except:
 			if verboseflag: print 'cannot create lock file %s' % LFtmp
-			raise 'cannot create lock file %s' % LFtmp
+			raise IOError, 'cannot create lock file %s' % LFtmp
 			return
+		if verboseflag: print 'createing lock file %s' % LFtmp
 		lf.write("%10s\n" % pid)
 		lf.close()
 		while 1:
 			try:
-				os.link(LFtmp,self.LF)
+				os.link(LFtmp, self.LF)
 			except:
-				if verboseflag: print "lock: cannot link %s to %s" % (LFtmp, self.LF)
+				if verboseflag: print "lock: cannot link %s to %s: %s" % (LFtmp, self.LF, sys.exc_info())
 			else:
 				if verboseflag: print "lock: pid %s locked %s" % (pid, self.LF)
 				os.remove(LFtmp)
@@ -202,7 +203,7 @@ class Uulock:
 				self.LF=""
 				os.remove(LFtmp)
 				if verboseflag: print "lock: (%s) in use" % resource
-				raise "lock (%s) in use" % resource
+				raise IOError, "lock (%s) in use" % resource
 				break
 		
 
@@ -283,7 +284,6 @@ class clientC:
 	self.fs={}
 	self.fslist=[]
 	self.exclude=[]
-	self.uulock=None
 
   def addfs(self, fs):
 	self.fs[fs.name]=fs
@@ -309,18 +309,6 @@ class clientC:
 		a+='	fs "%s"%s;\n' % (fsys, str(self.fs[fsys]))
 	return a
 
-  def Lock(self):
-	if self.uulock:		#NB. already locked 
-		return
-	if self.bkpserver:
-		lockdir=self.bkppath
-	else:
-		lockdir=general.bkppath
-	self.uulock=Uulock(os.getpid(), self.name, lockdir+"/LCK")
-
-
-  def UnLock(self):
-	self.uulock.unlock()
 
 #
 #
@@ -896,9 +884,7 @@ def runjob(threadno):
 	while taskQ.qsize():
 		client, cmd=taskQ.get(1)
 
-		cmd['v']=3+verboseflag
-		if cmd['v'] > 9:
-			cmd['v']=9
+		cmd['v']=min(3+verboseflag,9)
 
 		if clients[client].wakeup:
 			logprint("%s   waking client\n" % timestamp())
@@ -907,6 +893,19 @@ def runjob(threadno):
 			time.sleep(5)
 			clients[client].wakeup=0
 
+		lockdir="%(srvdir)s" % cmd
+		lockstr="%(client)s-%(fs)s" % cmd
+		lockstr=lockstr.replace("/","-")
+		try:
+			uulock=Uulock(os.getpid(), lockstr, lockdir+"/LCK")
+		except IOError, lockfailreason:
+			cmd['lockfailreason']=lockfailreason
+			res=[[timestamp(), "Cannot lock client %(client)s, fs %(fs)s: %s(lockfailreason)s\n" % cmd]]
+			err=[[timestamp(), "fs not backed up\n"]]
+			code=1
+			resultQ.put([cmd, res, err, code])
+			continue
+
 		if cmd['cmd'] == 'rdiff':
 			exc=""
 			for ex in cmd['excludelist']:
@@ -932,8 +931,11 @@ def runjob(threadno):
 		if verboseflag:
 			logprint("%s Starting thread %d for %s:%s\n" % (timestamp(), threadno, cmd['client'], cmd['fs']))
 			logflush()
-		res,err,code=runcmd(client, cmd['fs'], "%s %s" % (cmdbin, cmdarg),ts=1)
-		resultQ.put([cmd, res,err,code])
+		res,err,code=runcmd(client, cmd['fs'], "%s %s" % (cmdbin, cmdarg), ts=1)
+		resultQ.put([cmd, res, err, code])
+
+		uulock.unlock()
+
 	if verboseflag:
 		logprint("%s Thread %d done\n" % (timestamp(), threadno))
 
@@ -941,7 +943,6 @@ def runjob(threadno):
 # Main backup function
 #
 def do_backup():
-	lockedClients=[]
 	if not ourserver:
 		logprint("This server (%s) not listed in config's server section\n" % ourhostname)
 		logprint("Nothing to do, exiting\n")
@@ -1019,9 +1020,6 @@ def do_backup():
 				cmd[fs]['cmd']='rdiff'
 
 			taskQ.put([client, cmd[fs]])
-			if not client in lockedClients:
-				clients[client].Lock()
-				lockedClients+=[client]
 	jobs=taskQ.qsize()
 	threads=min(jobs, ourserver.queues)
 	if verboseflag:
@@ -1110,8 +1108,6 @@ def do_backup():
 			logprint("\n")
 
 	endtime=run_cleanup(cleanupdirs)
-	for client in lockedClients:
-		clients[client].UnLock()
 
 	try:
 		x=os.statvfs('.')
